/*
 *  Copyright (C) 2003, 2006, 2008, 2009 Apple Inc. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 */

#ifndef ScopeChainMark_h
#define ScopeChainMark_h

#include "ScopeChain.h"

namespace JSC {

    inline void ScopeChain::markAggregate(MarkStack& markStack) const
    {
        for (ScopeChainNode* n = m_node; n; n = n->next)
            markStack.append(n->object);
    }

} // namespace JSC

#endif // ScopeChainMark_h
